@@ -15,44 +15,46 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; If not, see <http://www.gnu.org/licenses/>.
 
-import logging
 from typing import Any
 from typing import Optional
 
-from gi.repository import GLib
-from gi.repository import Gio
+import logging
+from collections.abc import Sequence
 
-from nbxmpp.namespaces import Namespace
+from gi.repository import Gio
+from gi.repository import GLib
+
+from nbxmpp.addresses import NoMoreAddresses
+from nbxmpp.addresses import ServerAddresses
+from nbxmpp.const import ConnectionProtocol
+from nbxmpp.const import ConnectionType
+from nbxmpp.const import Mode
+from nbxmpp.const import StreamError
+from nbxmpp.const import StreamState
+from nbxmpp.dispatcher import StanzaDispatcher
+from nbxmpp.errors import CancelledError
+from nbxmpp.errors import StanzaError
+from nbxmpp.errors import TimeoutStanzaError
 from nbxmpp.http import HTTPSession
-from nbxmpp.protocol import Features
-from nbxmpp.protocol import StanzaMalformed
-from nbxmpp.protocol import SessionRequest
+from nbxmpp.namespaces import Namespace
 from nbxmpp.protocol import BindRequest
-from nbxmpp.protocol import TLSRequest
+from nbxmpp.protocol import Features
 from nbxmpp.protocol import isResultNode
 from nbxmpp.protocol import JID
 from nbxmpp.protocol import Protocol
+from nbxmpp.protocol import SessionRequest
+from nbxmpp.protocol import StanzaMalformed
+from nbxmpp.protocol import TLSRequest
 from nbxmpp.protocol import WebsocketCloseHeader
-from nbxmpp.errors import TimeoutStanzaError
-from nbxmpp.errors import StanzaError
-from nbxmpp.errors import CancelledError
-from nbxmpp.addresses import ServerAddresses
-from nbxmpp.addresses import NoMoreAddresses
-from nbxmpp.tcp import TCPConnection
-from nbxmpp.websocket import WebsocketConnection
-from nbxmpp.smacks import Smacks
 from nbxmpp.sasl import SASL
-from nbxmpp.const import StreamState
-from nbxmpp.const import StreamError
-from nbxmpp.const import ConnectionType
-from nbxmpp.const import ConnectionProtocol
-from nbxmpp.const import Mode
-from nbxmpp.dispatcher import StanzaDispatcher
-from nbxmpp.util import get_stream_header
+from nbxmpp.smacks import Smacks
+from nbxmpp.tcp import TCPConnection
 from nbxmpp.util import generate_id
+from nbxmpp.util import get_stream_header
+from nbxmpp.util import LogAdapter
 from nbxmpp.util import Observable
 from nbxmpp.util import validate_stream_header
-from nbxmpp.util import LogAdapter
+from nbxmpp.websocket import WebsocketConnection
 
 log = logging.getLogger('nbxmpp.stream')
 
@@ -124,6 +126,8 @@ class Client(Observable):
         self._remote_address = None
         self._mode = Mode.CLIENT
 
+        self._fallback_ns: set[str] = set()
+
         self._ping_source_id = None
         self._tasks = []
 
@@ -303,10 +307,10 @@ class Client(Observable):
 
     @property
     def mechs(self):
-        return set(self._allowed_mechs or set(['SCRAM-SHA-512',
-                                               'SCRAM-SHA-256',
-                                               'SCRAM-SHA-1',
-                                               'PLAIN']))
+        return set(self._allowed_mechs or {'SCRAM-SHA-512',
+                                           'SCRAM-SHA-256',
+                                           'SCRAM-SHA-1',
+                                           'PLAIN'})
 
     def set_mechs(self, mechs):
         self._allowed_mechs = mechs
@@ -356,6 +360,12 @@ class Client(Observable):
     def _set_bound_jid(self, jid):
         self._jid = JID.from_string(jid)
 
+    def set_supported_fallback_ns(self, ns: Sequence[str]) -> None:
+        self._fallback_ns = set(ns)
+
+    def get_supported_fallback_ns(self) -> set[str]:
+        return set(self._fallback_ns)
+
     @property
     def has_error(self):
         return self._error[0] is not None
@@ -669,14 +679,14 @@ class Client(Observable):
                 # other connection methods if an error happensafterwards
                 self._connect_successful = True
 
+            self.state = StreamState.WAIT_FOR_FEATURES
+
+        elif self.state == StreamState.WAIT_FOR_FEATURES:
             if self._stream_authenticated and self._mode.is_login_test:
                 self.notify('login-successful')
                 self.disconnect()
                 return
 
-            self.state = StreamState.WAIT_FOR_FEATURES
-
-        elif self.state == StreamState.WAIT_FOR_FEATURES:
             if stanza.getName() != 'features':
                 self._log.error('Invalid response: %s', stanza)
                 self._disconnect_with_error(
